# -*- coding: utf-8 -*-
"""FileSize class."""

from typing import Union, NamedTuple


class FileSize:
    """Data type for file sizes with value and unit."""

    __slots__ = ('_value', '_unit')

    Multiples = {
        'Byt': 1,
        'KiB': 1024,
        'MiB': 1024 ** 2,
        'GiB': 1024 ** 3,
        'TiB': 1024 ** 4,
        'PiB': 1024 ** 5,
        'EiB': 1024 ** 6,
        'ZiB': 1024 ** 7,
        'YiB': 1024 ** 8,
        }

    def __init__(self,
                 size: Union[int, float],
                 unit: str = 'Byt'):

        if not FileSize.Multiples.get(unit):
            raise ValueError(f"Unknown value '{unit}' for parameter 'unit'")

        if unit == 'Byt':
            if size % 1:
                raise ValueError(
                    "Parameter 'size' must be a multiple of 1 Byte")

        if not size % 1:
            size = int(size)

        self._value = size
        self._unit = unit

        # Parameter 'size' must not be less than 1 byte
        if size * FileSize.Multiples.get(unit) < 1:
            print("Parameter 'size' must not be less than 1 byte.")
            self._value = None

    @property
    def value(self) -> Union[int, float]:
        """Get filesize value."""
        return self._value

    @property
    def unit(self) -> str:
        """Get filesize unit."""
        return self._unit

    @property
    def namedtuple(self) -> NamedTuple:
        """Get filesize as named tuple."""
        nt = NamedTuple('filesize', [('size', Union[int, float]),
                                     ('unit', str)])
        return nt(self._value, self._unit)

    def __str__(self):
        if self._value is None:
            return ""

        return f"{self._value} {self._unit}"

    def __iter__(self):
        for item in (self._value, self._unit):
            yield item

    # Arithmetic operators
    def __add__(self, other):
        if isinstance(other, FileSize):
            result = self._value * FileSize.Multiples.get(self._unit)
            result += other._value * FileSize.Multiples.get(other._unit)
            return FileSize(size=result, unit='Byt')
        return NotImplemented

    def __sub__(self, other):
        if isinstance(other, FileSize):
            result = self._value * FileSize.Multiples.get(self._unit)
            result -= other._value * FileSize.Multiples.get(other._unit)
            return FileSize(size=result, unit='Byt')
        return NotImplemented

    def __mul__(self, other: Union[int, float]):
        if isinstance(other, (int, float)):
            result = self._value * other
            return FileSize(size=result, unit=self._unit)
        return NotImplemented

    def __rmul__(self, other: Union[int, float]):
        if isinstance(other, (int, float)):
            result = self._value * other
            return FileSize(size=result, unit=self._unit)
        return NotImplemented

    def __truediv__(self, other: Union[int, float]):
        if isinstance(other, (int, float)):
            result = self._value / other
            return FileSize(size=result, unit=self._unit)
        return NotImplemented

    def __pow__(self, other: Union[int, float]):
        if isinstance(other, (int, float)):
            result = self._value ** other
            return FileSize(size=result, unit=self._unit)
        return NotImplemented

    # Comparison operators
    def __eq__(self, other):
        if isinstance(other, FileSize):
            return self._value == other._value
        return NotImplemented

    def __ne__(self, other):
        if isinstance(other, FileSize):
            return self._value != other._value
        return NotImplemented

    def __lt__(self, other):
        if isinstance(other, FileSize):
            return self._value < other._value
        return NotImplemented

    def __le__(self, other):
        if isinstance(other, FileSize):
            return self._value <= other._value
        return NotImplemented

    def __gt__(self, other):
        if isinstance(other, FileSize):
            return self._value > other._value
        return NotImplemented

    def __ge__(self, other):
        if isinstance(other, FileSize):
            return self._value >= other._value
        return NotImplemented

    # Custom methods
    def convert(self, unit: str = 'auto', max_decimals: int = -1):
        r"""
        Convert FileSize unit to another FileSize unit.

        Parameters
        ----------
        unit : str, optional
            New unit to convert to. Valid parameters are: \n
            ['auto', 'Byt', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'] \n
            The default is 'auto'.
        max_decimals : int, optional
            max_decimals == -1; No changes of result\n
            max_decimals == 0; No decimals in result\n
            max_decimals > 0; Rounded result to given maximum decimals\n
            The default is -1.

        Raises
        ------
        ValueError
            Invalid value for parameter 'unit'.

        Returns
        -------
        FileSize
            A new instance of FileSize.

        """
        value = self._value * FileSize.Multiples.get(self._unit)

        if unit == 'auto':
            if value < FileSize.Multiples.get('KiB'):
                size = value
                unit = 'Byt'
            elif value < FileSize.Multiples.get('MiB'):
                size = value / FileSize.Multiples.get('KiB')
                unit = 'KiB'
            elif value < FileSize.Multiples.get('GiB'):
                size = value / FileSize.Multiples.get('MiB')
                unit = 'MiB'
            elif value < FileSize.Multiples.get('TiB'):
                size = value / FileSize.Multiples.get('GiB')
                unit = 'GiB'
            elif value < FileSize.Multiples.get('PiB'):
                size = value / FileSize.Multiples.get('TiB')
                unit = 'TiB'
            elif value < FileSize.Multiples.get('EiB'):
                size = value / FileSize.Multiples.get('PiB')
                unit = 'PiB'
            elif value < FileSize.Multiples.get('ZiB'):
                size = value / FileSize.Multiples.get('EiB')
                unit = 'EiB'
            elif value < FileSize.Multiples.get('YiB'):
                size = value / FileSize.Multiples.get('ZiB')
                unit = 'ZiB'
            else:
                size = value / FileSize.Multiples.get('YiB')
                unit = 'YiB'
        elif FileSize.Multiples.get(unit):
            size = value / FileSize.Multiples.get(unit)
        else:
            raise ValueError("Invalid value for parameter 'unit'")

        if max_decimals == 0:
            # No decimals
            size = round(size)
        if max_decimals > 0:
            size = round(size, max_decimals)

        return FileSize(size, unit)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'targetwidgetrSNmJf.ui'
##
## Created by: Qt User Interface Compiler version 6.4.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QFrame,
    QHBoxLayout, QLabel, QLayout, QProgressBar,
    QPushButton, QSizePolicy, QSpacerItem, QVBoxLayout,
    QWidget)

class Ui_targetwidget(object):
    def setupUi(self, targetwidget):
        if not targetwidget.objectName():
            targetwidget.setObjectName(u"targetwidget")
        targetwidget.resize(500, 123)
        self.verticalLayout_2 = QVBoxLayout(targetwidget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.frame_targetwidget = QFrame(targetwidget)
        self.frame_targetwidget.setObjectName(u"frame_targetwidget")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame_targetwidget.sizePolicy().hasHeightForWidth())
        self.frame_targetwidget.setSizePolicy(sizePolicy)
        self.frame_targetwidget.setFrameShape(QFrame.StyledPanel)
        self.frame_targetwidget.setFrameShadow(QFrame.Raised)
        self.horizontalLayout = QHBoxLayout(self.frame_targetwidget)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_settings = QHBoxLayout()
        self.horizontalLayout_settings.setObjectName(u"horizontalLayout_settings")
        self.checkBox_switch = QCheckBox(self.frame_targetwidget)
        self.checkBox_switch.setObjectName(u"checkBox_switch")
        self.checkBox_switch.setChecked(True)

        self.horizontalLayout_settings.addWidget(self.checkBox_switch)

        self.label_stickname = QLabel(self.frame_targetwidget)
        self.label_stickname.setObjectName(u"label_stickname")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.label_stickname.sizePolicy().hasHeightForWidth())
        self.label_stickname.setSizePolicy(sizePolicy1)
        self.label_stickname.setAlignment(Qt.AlignCenter)

        self.horizontalLayout_settings.addWidget(self.label_stickname)


        self.verticalLayout.addLayout(self.horizontalLayout_settings)

        self.horizontalLayout_progress = QHBoxLayout()
        self.horizontalLayout_progress.setObjectName(u"horizontalLayout_progress")
        self.progressbar = QProgressBar(self.frame_targetwidget)
        self.progressbar.setObjectName(u"progressbar")
        self.progressbar.setValue(0)

        self.horizontalLayout_progress.addWidget(self.progressbar)


        self.verticalLayout.addLayout(self.horizontalLayout_progress)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.checkBox_verify = QCheckBox(self.frame_targetwidget)
        self.checkBox_verify.setObjectName(u"checkBox_verify")

        self.horizontalLayout_2.addWidget(self.checkBox_verify)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.horizontalLayout_stats = QHBoxLayout()
        self.horizontalLayout_stats.setSpacing(12)
        self.horizontalLayout_stats.setObjectName(u"horizontalLayout_stats")
        self.label_speed = QLabel(self.frame_targetwidget)
        self.label_speed.setObjectName(u"label_speed")

        self.horizontalLayout_stats.addWidget(self.label_speed)

        self.label_time_cpu = QLabel(self.frame_targetwidget)
        self.label_time_cpu.setObjectName(u"label_time_cpu")

        self.horizontalLayout_stats.addWidget(self.label_time_cpu)

        self.label_time = QLabel(self.frame_targetwidget)
        self.label_time.setObjectName(u"label_time")

        self.horizontalLayout_stats.addWidget(self.label_time)


        self.horizontalLayout_2.addLayout(self.horizontalLayout_stats)


        self.verticalLayout.addLayout(self.horizontalLayout_2)


        self.horizontalLayout.addLayout(self.verticalLayout)

        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setSizeConstraint(QLayout.SetDefaultConstraint)
        self.label_sticksize = QLabel(self.frame_targetwidget)
        self.label_sticksize.setObjectName(u"label_sticksize")
        self.label_sticksize.setAlignment(Qt.AlignCenter)

        self.verticalLayout_3.addWidget(self.label_sticksize)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.pushButton_stop = QPushButton(self.frame_targetwidget)
        self.pushButton_stop.setObjectName(u"pushButton_stop")
        icon = QIcon()
        iconThemeName = u"media-playback-stop"
        if QIcon.hasThemeIcon(iconThemeName):
            icon = QIcon.fromTheme(iconThemeName)
        else:
            icon.addFile(u"../../../../../../.designer/backup", QSize(), QIcon.Normal, QIcon.Off)

        self.pushButton_stop.setIcon(icon)

        self.horizontalLayout_4.addWidget(self.pushButton_stop)

        self.pushButton_start = QPushButton(self.frame_targetwidget)
        self.pushButton_start.setObjectName(u"pushButton_start")
        icon1 = QIcon()
        iconThemeName = u"media-playback-start"
        if QIcon.hasThemeIcon(iconThemeName):
            icon1 = QIcon.fromTheme(iconThemeName)
        else:
            icon1.addFile(u"../../../../../../.designer/backup", QSize(), QIcon.Normal, QIcon.Off)

        self.pushButton_start.setIcon(icon1)

        self.horizontalLayout_4.addWidget(self.pushButton_start)


        self.verticalLayout_3.addLayout(self.horizontalLayout_4)

        self.comboBox_buffers = QComboBox(self.frame_targetwidget)
        self.comboBox_buffers.setObjectName(u"comboBox_buffers")
        sizePolicy2 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.comboBox_buffers.sizePolicy().hasHeightForWidth())
        self.comboBox_buffers.setSizePolicy(sizePolicy2)

        self.verticalLayout_3.addWidget(self.comboBox_buffers)


        self.horizontalLayout.addLayout(self.verticalLayout_3)


        self.verticalLayout_2.addWidget(self.frame_targetwidget)


        self.retranslateUi(targetwidget)
    # setupUi

    def retranslateUi(self, targetwidget):
        targetwidget.setWindowTitle(QCoreApplication.translate("targetwidget", u"Form", None))
#if QT_CONFIG(tooltip)
        self.checkBox_switch.setToolTip(QCoreApplication.translate("targetwidget", u"Enable/disable writing", None))
#endif // QT_CONFIG(tooltip)
        self.checkBox_switch.setText("")
        self.label_stickname.setText(QCoreApplication.translate("targetwidget", u"USB-Stick Info", None))
#if QT_CONFIG(tooltip)
        self.checkBox_verify.setToolTip(QCoreApplication.translate("targetwidget", u"Verify after writing (Hash).", None))
#endif // QT_CONFIG(tooltip)
        self.checkBox_verify.setText(QCoreApplication.translate("targetwidget", u"Verify", None))
#if QT_CONFIG(tooltip)
        self.label_speed.setToolTip(QCoreApplication.translate("targetwidget", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"li.unchecked::marker { content: \"\\2610\"; }\n"
"li.checked::marker { content: \"\\2612\"; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Overall speed after writing.</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.label_speed.setText(QCoreApplication.translate("targetwidget", u"0.00 Mib/s", None))
#if QT_CONFIG(tooltip)
        self.label_time_cpu.setToolTip(QCoreApplication.translate("targetwidget", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"li.unchecked::marker { content: \"\\2610\"; }\n"
"li.checked::marker { content: \"\\2612\"; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">CPU time</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">(without verification)</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.label_time_cpu.setText(QCoreApplication.translate("targetwidget", u"0.00 s", None))
#if QT_CONFIG(tooltip)
        self.label_time.setToolTip(QCoreApplication.translate("targetwidget", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"li.unchecked::marker { content: \"\\2610\"; }\n"
"li.checked::marker { content: \"\\2612\"; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Progress time</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">(without verification)</p></body></html>", None))
#endif // QT_CONFIG(tooltip)
        self.label_time.setText(QCoreApplication.translate("targetwidget", u"0.00 s", None))
#if QT_CONFIG(tooltip)
        self.label_sticksize.setToolTip(QCoreApplication.translate("targetwidget", u"USB stick size", None))
#endif // QT_CONFIG(tooltip)
        self.label_sticksize.setText(QCoreApplication.translate("targetwidget", u"0 GB", None))
#if QT_CONFIG(tooltip)
        self.pushButton_stop.setToolTip(QCoreApplication.translate("targetwidget", u"Stop", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.pushButton_start.setToolTip(QCoreApplication.translate("targetwidget", u"Start", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.comboBox_buffers.setToolTip(QCoreApplication.translate("targetwidget", u"Write buffer", None))
#endif // QT_CONFIG(tooltip)
    # retranslateUi




# -*- coding: utf-8 -*-
"""Threading."""

import os
import hashlib
from pathlib import Path

from gi.repository import GLib, Gio
from PySide6.QtCore import Signal, Slot, QThread

from usb_imager.shared_objects import SharedObjects as shared
from usb_imager.modules.timer import Timer


class WritingThread(QThread):
    """Writing worker class.

    Unlike QRunnable the QThread class does inherit from QObject so you can
    define the signals on the thread object itself.
    """

    sig_progress_value = Signal(int)
    sig_progress_format = Signal(str)
    sig_timer_results = Signal(float, float, int)
    sig_status = Signal(str)

    timer = Timer()

    def __init__(self,
                 image_file: Path,
                 object_path: str,
                 bufsize: int = 1024 ** 2,
                 validation: bool = False):
        super().__init__()

        self.image_file = image_file
        self.image_size = os.path.getsize(self.image_file)
        self.object_path = object_path
        self.bufsize_user = bufsize
        self.validation = validation

        self.cancel = False
        self.progressbar_current_value = 0

    def stop_writing(self) -> None:
        """Stop writing to device."""
        self.cancel = True

    @Slot()
    def run(self):
        """Run thread."""
        written = 0
        progress_steps = 100

        in_fd, out_fd = self.get_fd_all()
        if not in_fd or not out_fd:
            return

        try:
            self.sig_progress_format.emit('Writing... %p%')
            self.sig_timer_results.emit(0, 0, 0)
            self.timer.start()

            while written < self.image_size:
                if self.cancel is True:
                    break
                # NOTE:
                # os.sendfile doesn't work in my tests with parameter less than
                # count=4096
                written += os.sendfile(out_fd, in_fd,
                                       offset=None, count=self.bufsize_user)

                if self.validation:
                    progress_steps = 50
                self.update_progressbar(written,
                                        self.image_size,
                                        self.progressbar_current_value,
                                        progress_start=0,
                                        progress_steps=progress_steps)

            self.timer.stop()
            bytes_per_sec = int(written / self.timer.result)
            self.sig_timer_results.emit(self.timer.result,
                                        self.timer.result_cpu,
                                        bytes_per_sec)

        except OSError:
            # FIXME
            # Here is not always thrown an exception.
            # (e.g. when removing a USB stick while writing).
            format_msg = 'Error while writing, writing canceled... %p%'
            self.sig_progress_format.emit(format_msg)
        else:
            if self.cancel is True:
                format_msg = 'Writing canceled... %p%'
            else:
                format_msg = 'Writing finished... %p%'
            self.sig_progress_format.emit(format_msg)
        finally:
            self.close_fd_all(in_fd, out_fd)

        self.close_fd_all(in_fd, out_fd)

        if self.validation:
            format_msg = 'Validating... %p%'
            self.sig_progress_format.emit(format_msg)

            is_valid = self.start_validation()

            if self.cancel is True:
                format_msg = 'Validation canceled... %p%'
                self.sig_progress_format.emit(format_msg)
            elif is_valid is True:
                format_msg = 'Validation successful... %p%'
                self.sig_progress_format.emit(format_msg)
            elif is_valid is False:
                format_msg = 'Validation failed... %p%'
                self.sig_progress_format.emit(format_msg)

    def get_fd_all(self) -> tuple:
        in_fd = self.get_fd_image(self.image_file)
        out_fd = self.get_fd_device(mode='w')

        if not out_fd:
            os.close(in_fd)
            in_fd = None

        return in_fd, out_fd

    def get_fd_image(self, file: Path) -> int | None:
        filedescriptor = None

        try:
            filedescriptor = os.open(file, os.O_RDONLY)
        except OSError as error:
            self.sig_status.emit(error.args)

        return filedescriptor

    def get_fd_device(self, mode: str) -> int | None:
        filedescriptor = None

        if mode == 'r':
            arg_options = shared.udisks2.preset_args['open_device_backup']
        elif mode == 'w':
            arg_options = shared.udisks2.preset_args['open_device_restore']
        else:
            self.sig_status.emit('Couldn´t get a file descriptor for device.')
            return filedescriptor

        interface_block = \
            shared.udisks2.get_interface(self.object_path, 'Block')

        if not interface_block:
            return None

        # interface.call_open_device is recommended since UDisks v273
        if shared.udisks2.version >= 273:
            try:
                result = interface_block.call_open_device_sync(
                    mode, arg_options, Gio.UnixFDList(), None)
            except GLib.Error as error:
                self.sig_status.emit(error.message)
            else:
                if result:
                    out_fd, out_fd_list = result
                    filedescriptor = out_fd_list.get(out_fd.unpack())
                shared.udisks2.settle()

        return filedescriptor

    def close_fd_all(self, *args) -> bool:
        all_fd_closed = False

        try:
            for file_fd in args:
                os.close(file_fd)
        except OSError as error:
            self.sig_status.emit(error.args)
        else:
            all_fd_closed = True

        return all_fd_closed

    def start_validation(self) -> bool:
        valid = False

        hash_file = self.hash_file()
        hash_device = self.hash_device()

        if hash_file and hash_device:
            if hash_file == hash_device:
                valid = True

            # print('hash_file =', hash_file)
            # print('hash_device =', hash_device)

        return valid

    def hash_file(self) -> str:
        hexdigest = ''

        hashobj = hashlib.sha256()
        bufsize = 1024 ** 2
        read_total = 0

        with self.image_file.open('rb') as file:
            buffer = file.read(bufsize)
            hashobj.update(buffer)
            read_total += len(buffer)
            while len(buffer) > 0:
                if self.cancel is True:
                    break
                buffer = file.read(bufsize)
                hashobj.update(buffer)
                read_total += len(buffer)
                self.update_progressbar(read_total,
                                        self.image_size,
                                        self.progressbar_current_value,
                                        progress_start=50,
                                        progress_steps=25)

        hexdigest = hashobj.hexdigest()

        return hexdigest

    def hash_device(self) -> str:
        hexdigest = ''

        hashobj = hashlib.sha256()
        bufsize = 1024 ** 2
        bufsize_rest = self.image_size % bufsize
        read = 0

        fd_in = self.get_fd_device('r')
        if fd_in:
            if bufsize_rest:
                buffer = os.read(fd_in, bufsize_rest)
                hashobj.update(buffer)
                read += len(buffer)
            while read < self.image_size:
                if self.cancel is True:
                    break
                buffer = os.read(fd_in, bufsize)
                hashobj.update(buffer)
                read += len(buffer)
                self.update_progressbar(
                    read,
                    self.image_size,
                    self.progressbar_current_value,
                    progress_start=75,
                    progress_steps=25)
            self.close_fd_all(fd_in)

        hexdigest = hashobj.hexdigest()

        return hexdigest

    def update_progressbar(self,
                           progress: int,
                           progress_total: int,
                           progressbar_current_value: int,
                           progress_start: int = 0,
                           progress_steps: int = 100) -> int:
        """
        Update the progressbar.

        Parameters
        ----------
        progress : int
            The processed size of the progress object.
        progress_total : int
            The total size of the progress object.
        progressbar_current_value : int
            The current value of progressbar.

            This function needs to know the current value of the progress bar
            to update it only when it changes. The calculated floating point
            values are truncated to their nominal value for
            QProgressBar.setValue (int). Therefore they often result in the
            same value.

            If the current value cannot be read from QProgressBar.value(), it
            can also be stored in a variable of the calling function of
            update_progressbar().
        progress_start : int, optional
            The starting value of the progress bar. The default is 0.
        progress_steps : int, optional
            The end value of the progress bar. The default is 100.

        Returns
        -------
        int
            The current value of progressbar.

        """
        progressbar_value = int(
            progress * progress_steps / progress_total + progress_start)
        if progressbar_value > progressbar_current_value:
            self.sig_progress_value.emit(progressbar_value)
            progressbar_current_value = progressbar_value



# -*- coding: utf-8 -*-
"""ui_application."""

import os
import re
from pathlib import Path
import importlib.resources

from gi.repository import GLib
from PySide6.QtGui import QIcon
from PySide6.QtCore import QMetaObject, Slot
from PySide6.QtWidgets import (
    QApplication, QMainWindow, QWidget, QFileDialog, QMessageBox, QLabel
)

from usb_imager.appinfo import (
    APP_NAME, APP_VERSION, APP_ORGANISATION, APP_ID, APP_TITLE, APP_ABOUT,
)
from usb_imager.ui_mainwindow import Ui_MainWindow
from usb_imager.ui_targetwidget import Ui_targetwidget
from usb_imager.shared_objects import SharedObjects as shared
from usb_imager.threads import WritingThread
from usb_imager.modules.filesize import FileSize


STATUSBAR_TIMEOUT = 5000


class Application(QApplication):
    """Application."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.setup_application()
        self.mainwindow = MainWindow()
        self.mainwindow.show()

    def setup_application(self) -> None:
        """Application setup."""
        self.setApplicationName(APP_NAME)
        self.setApplicationVersion(APP_VERSION)
        self.setOrganizationName(APP_ORGANISATION)
        self.setOrganizationDomain(APP_ID)

        icon = get_app_icon()
        self.setWindowIcon(icon)

        # TODO
        # Configure style
        # self.setStyle('kvantum')
        # Choose theme: KvLibadwaitaMaiaDark in kvantum configurator

        # app_style = get_qss_text('style.qss')
        # self.setStyleSheet(app_style)


class MainWindow(QMainWindow):
    """MainWidget."""

    def __init__(self):
        super().__init__()

        self.init_mainwindow()

        shared.udisks2.signal_connect_obj_added(self.callback_object_added)
        shared.udisks2.signal_connect_obj_removed(self.callback_object_removed)

        self.init_targetwidgets()

    def init_mainwindow(self) -> None:
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.setWindowTitle(APP_TITLE)
        self.ui.toolButton_menu.setMenu(self.ui.menuInfo)
        self.ui.menubar.hide()

        self.label_version = QLabel('V ' + APP_VERSION)
        self.ui.statusbar.addPermanentWidget(self.label_version)

        QMetaObject.connectSlotsByName(self)

    def init_targetwidgets(self) -> None:
        block_device_pathes = \
            shared.udisks2.get_object_pathes('block_devices')

        for object_path in block_device_pathes:
            if self.is_usb_key(object_path):
                self.add_targetwidget(object_path)

    def add_targetwidget(self, object_path: str) -> str:
        widget = TargetWidget(object_path, self)
        widget.ui.progressbar.valueChanged.connect(
            self.update_progressbar_all)
        # widget.ui.pushButton_stop

        self.ui.verticalLayout_targets.addWidget(widget)
        shared.targetwidgets.append(widget)

        return widget.device_info

    def callback_object_added(self, sender, dbus_object) -> None:
        object_path = dbus_object.get_object_path()

        if self.is_usb_key(object_path):
            device_info = self.add_targetwidget(object_path)
            self.ui.statusbar.showMessage(
                f"USB-Device added:  {device_info}", STATUSBAR_TIMEOUT)

    def callback_object_removed(self, sender, dbus_object) -> None:
        object_path = dbus_object.get_object_path()

        for widget in shared.targetwidgets:
            if widget.object_path == object_path:
                device_info = widget.device_info
                widget.close()
                shared.targetwidgets.remove(widget)
                self.ui.statusbar.showMessage(
                    f"USB-Device removed:  {device_info}", STATUSBAR_TIMEOUT)

    @Slot(bool)
    def on_checkBox_select_all_toggled(self, checked: bool) -> None:
        if shared.targetwidgets:
            for widget in shared.targetwidgets:
                widget.ui.checkBox_switch.setChecked(checked)

    @Slot(bool)
    def on_checkBox_verify_all_toggled(self, checked: bool) -> None:
        if shared.targetwidgets:
            for widget in shared.targetwidgets:
                widget.ui.checkBox_verify.setChecked(checked)

    @Slot()
    def on_pushButton_open_clicked(self):
        homedir = Path.home() / 'Downloads'
        filters = ('Image Files (*.iso *.img *.raw *.bin *.dd)',)

        filename = QFileDialog.getOpenFileName(
            parent=self,
            caption="Open Image",
            dir=str(homedir),
            filter=';;'.join(filters)
            )[0]

        if filename:
            filename = Path(filename)
            if filename.match('*.iso') and not self.is_bootable_iso(filename):
                return
            shared.imagepath = filename
            self.ui.label_imagename.setText(shared.imagepath.name)

            size = os.path.getsize(shared.imagepath)
            imagesize = FileSize(size).convert(max_decimals=1)
            self.ui.label_imagesize.setText(str(imagesize))

    @Slot()
    def on_pushButton_start_all_clicked(self) -> None:
        if not shared.targetwidgets:
            return

        for widget in shared.targetwidgets:
            if widget.ui.checkBox_switch.isChecked():
                break
        else:
            message = "Please select a device for writing."
            self.ui.statusbar.showMessage(message, STATUSBAR_TIMEOUT)
            return

        if not shared.imagepath:
            message = "Please select an image to use!"
            self.ui.statusbar.showMessage(message, STATUSBAR_TIMEOUT)
            return

        # Check for mounted filesystems on all devices and try to unmount them
        for widget in shared.targetwidgets:
            widget.are_filesystem_mounted()

        # Security question before start writing
        text = \
            "All selected devices will be completely overwritten "\
            "and data on it will be lost.\n\n" \
            "Do you want to continue the process?"
        result = QMessageBox.warning(self, "Warning", text,
                                     QMessageBox.Yes | QMessageBox.No)
        if result == QMessageBox.No:
            return

        # Start writing
        for widget in shared.targetwidgets:
            if not widget.ui.checkBox_switch.isChecked():
                continue
            widget.on_pushButton_start_clicked(
                security_message=False)

    @Slot()
    def on_pushButton_stop_all_clicked(self) -> None:
        if shared.targetwidgets:
            for widget in shared.targetwidgets:
                widget.on_pushButton_stop_clicked()

    @Slot()
    def on_actionAboutQt_triggered(self) -> None:
        QMessageBox.aboutQt(self)

    @Slot()
    def on_actionAbout_triggered(self) -> None:
        QMessageBox.about(self, "About USB-Imager", APP_ABOUT)

    def is_bootable_iso(self, filepath: Path) -> bool:
        is_bootable = False

        try:
            with filepath.open(mode='rb') as file:
                file.seek(510)
                mbr_signature = file.read(2).hex()
        except OSError as error:
            self.ui.statusbar.showMessage(error.args)
        else:
            if mbr_signature == '55aa':
                is_bootable = True
            else:
                message = "Selected ISO file is not bootable!"
                self.ui.statusbar.showMessage(message, STATUSBAR_TIMEOUT)

        return is_bootable

    def is_usb_key(self, object_path: str) -> bool:
        """
        Check if given object path is a USB-Stick.

        NOTE: This method only works if a device is plugged in!!!

        Parameters
        ----------
        object_path : str
            An object path string like \
                '/org/freedesktop/UDisks2/block_devices/sda'.

        Returns
        -------
        bool
            True/False if USB-Key is detected or not.

        """
        is_usbdev = True

        whole_block_device_pathes = \
            shared.udisks2.get_object_pathes_whole_block_device()
        interface = shared.udisks2.get_interface(object_path, 'Block')
        if not interface:
            return False

        if object_path not in whole_block_device_pathes:
            is_usbdev = False
        elif re.search(r"[0-9]$", object_path):
            is_usbdev = False
        else:
            drive_path, device_path, size = \
                interface.get_properties('drive', 'device', 'size')

            if not drive_path or drive_path == '/':
                is_usbdev = False
            elif not device_path or device_path == '/':
                is_usbdev = False
            elif not size or size == 0:
                is_usbdev = False
            else:
                interface = shared.udisks2.get_interface(drive_path, 'Drive')
                if not interface:
                    return False

                connection_bus, removable = \
                    interface.get_properties('connection-bus', 'removable')

                if removable is False or connection_bus != "usb":
                    is_usbdev = False

        return is_usbdev

    def update_progressbar_all(self) -> None:
        threads = 0
        progressbar_values = 0

        for widget in shared.targetwidgets:
            if widget.copythread and widget.copythread.isRunning():
                threads += 1
                progressbar_values += widget.ui.progressbar.value()
            elif widget.ui.progressbar.value() == 100:
                threads += 1
                progressbar_values += 100

        if threads:
            value = progressbar_values / threads
            self.ui.progressbar_all.setFormat('Process... %p%')
            self.ui.progressbar_all.setValue(value)
        else:
            self.ui.progressbar_all.setFormat('All canceled... %p%')

        if self.ui.progressbar_all.value() == 100:
            self.ui.progressbar_all.setFormat('All done... %p%')


class TargetWidget(QWidget):
    """Widget stores all data about target device."""

    def __init__(self,
                 object_path: str,
                 mainwindow: QMainWindow):
        super().__init__()

        self.object_path = object_path
        self.mainwindow = mainwindow

        self.ui = Ui_targetwidget()
        self.ui.setupUi(self)

        # Device properties
        self.path = None
        self.vendor = "[No Vendor]"
        self.model = "[No Model]"
        self.size = 0
        self.init_properties()
        self.humanreadable_size = \
            str(FileSize(self.size).convert(max_decimals=1))

        self.copythread = None

        self.init_ui()

    def init_properties(self) -> None:
        """Set properties path, vendor, model, size."""
        drive_path = \
            shared.udisks2.get_property(self.object_path, 'Block', 'drive')
        device_path = \
            shared.udisks2.get_property(self.object_path, 'Block', 'device')

        self.path = Path(device_path)

        interface = \
            shared.udisks2.get_interface(drive_path, 'Drive')
        self.size, vendor, model = \
            interface.get_properties('size', 'vendor', 'model')

        if vendor:
            self.vendor = vendor
        if model:
            self.model = model

    @property
    def device_info(self) -> str:
        return f"{self.vendor} - {self.model} ({self.path.name})"

    def init_ui(self) -> None:
        """Set USB-Stick informations to label_stickname."""

        self.ui.label_stickname.setText(self.device_info)
        self.ui.label_sticksize.setText(self.humanreadable_size)

        # Add buffer sizes to comboBox_buffers
        # NOTE: os.sendfile doesn't work in my test with parameter count < 4096
        sizes = [4096, 8192, 16384, 32768,
                 1024 ** 2, 2048 ** 2, 4096 ** 2]
        for blksize in sizes:
            filesize = str(FileSize(blksize).convert())
            self.ui.comboBox_buffers.addItem(filesize, blksize)
        self.ui.comboBox_buffers.setCurrentIndex(4)

        self.switch_visible_stats(False)
        QMetaObject.connectSlotsByName(self)

    @Slot()
    def on_pushButton_start_clicked(self,
                                    security_message: bool = True) -> None:

        if not self.ui.checkBox_switch.isChecked():
            # If checkBox_switch is not checked, pushButton_start is disabled
            # The return statement is nevertheless set for safety.
            return

        if not shared.imagepath:
            message = "Please select an image to use!"
            self.mainwindow.ui.statusbar.showMessage(
                message, STATUSBAR_TIMEOUT)
            return

        # Check if the target device is large enough for the image file.
        srcfile_size = os.path.getsize(shared.imagepath)
        if srcfile_size > self.size:
            message = \
                "Error:  " \
                "The size of the device is too small to write the image."
            self.mainwindow.ui.statusbar.showMessage(
                message, STATUSBAR_TIMEOUT)
            return

        # Check for mounted filesystems on all devices and try to unmount them
        if self.are_filesystem_mounted():
            return

        # Security question before start writing
        if security_message:
            text = \
                "The device will be completely overwritten " \
                "and data on it will be lost.\n\n" \
                "Do you want to continue the process?"
            result = QMessageBox.warning(self, "Warning", text,
                                         QMessageBox.Yes | QMessageBox.No)
            if result == QMessageBox.No:
                return

        # Start writing
        blksize = self.ui.comboBox_buffers.currentData()
        validation = self.ui.checkBox_verify.isChecked()

        self.copythread = WritingThread(shared.imagepath,
                                        self.object_path,
                                        blksize,
                                        validation)

        self.copythread.started.connect(
            lambda: self.switch_enabled_widgets(False))
        self.copythread.finished.connect(self.finished_writing)

        self.start_writing()

    @Slot()
    def on_pushButton_stop_clicked(self) -> None:
        if self.copythread:
            self.copythread.stop_writing()

    def start_writing(self) -> None:
        if self.copythread:
            self.copythread.sig_progress_value.connect(
                self.ui.progressbar.setValue)
            self.copythread.sig_progress_format.connect(
                self.ui.progressbar.setFormat)
            self.copythread.sig_timer_results.connect(
                self.set_stats)
            self.copythread.sig_status.connect(
                self.mainwindow.ui.statusbar.showMessage)

            self.mainwindow.ui.statusbar.clearMessage()
            self.copythread.start()

    def finished_writing(self) -> None:
        self.copythread.wait()
        self.switch_enabled_widgets(True)
        self.mainwindow.update_progressbar_all()

    def are_filesystem_mounted(self) -> bool:
        mounted = False

        mounted_filesystem_interfaces = \
            shared.udisks2.get_mounted_filesystems(self.object_path)

        if mounted_filesystem_interfaces:
            text = \
                f"There are filesystem(s) mounted on {self.device_info}.\n\n" \
                "Would you like to unmount them?"
            result = QMessageBox.warning(self, "Warning", text,
                                         QMessageBox.Yes | QMessageBox.No)
            if result == QMessageBox.No:
                mounted = True
                return mounted

            for iface_fs in mounted_filesystem_interfaces:
                # Unmount device
                arg_options = shared.udisks2.preset_args['unmount']
                try:
                    iface_fs.call_unmount_sync(arg_options)
                except GLib.Error as error:
                    self.mainwindow.ui.statusbar.showMessage(error.message)
                    mounted = True
                    break
                else:
                    shared.udisks2.settle()

        # TODO
                # Lock device if unlocked
                # object_path = iface_fs.get_object_path()
                # unlocked_encrypted = \
                #     shared.udisks2.get_unlocked_encrypted(object_path)
                # if unlocked_encrypted:
                #     arg_options = shared.udisks2.preset_args['lock']
                #     try:
                #         unlocked_encrypted.call_lock_sync(arg_options)
                #     except GLib.Error as error:
                #         self.mainwindow.ui.statusbar.showMessage(error.message)
                #         return

                #     shared.udisks2.settle()

        return mounted

    def switch_enabled_widgets(self, switch: bool) -> None:
        # TargetWidget widgets
        self.ui.pushButton_start.setEnabled(switch)
        self.ui.checkBox_switch.setEnabled(switch)
        self.ui.checkBox_verify.setEnabled(switch)
        self.ui.comboBox_buffers.setEnabled(switch)

        # MainWindow widgets

        # checkBox_switch and ui_pushButton_start_all remains disabled
        # as long as a copythread is running
        if switch is False:
            self.mainwindow.ui.pushButton_start_all.setEnabled(switch)
            self.mainwindow.ui.checkBox_select_all.setEnabled(switch)
            self.mainwindow.ui.checkBox_verify_all.setEnabled(switch)
        elif switch is True:
            for widget in shared.targetwidgets:
                if not widget.ui.checkBox_switch.isEnabled():
                    break
            else:
                self.mainwindow.ui.pushButton_start_all.setEnabled(switch)
                self.mainwindow.ui.checkBox_select_all.setEnabled(switch)
                self.mainwindow.ui.checkBox_verify_all.setEnabled(switch)

    def switch_visible_stats(self, switch: bool) -> None:
        self.ui.label_speed.setVisible(switch)
        self.ui.label_time_cpu.setVisible(switch)
        self.ui.label_time.setVisible(switch)

    @Slot(float, float, int)
    def set_stats(self,
                  time: float,
                  time_cpu: float,
                  bytes_per_sec: int) -> None:

        if not time and not time_cpu and not bytes_per_sec:
            self.switch_visible_stats(False)
            return

        if bytes_per_sec:
            speed = FileSize(bytes_per_sec).convert('MiB', max_decimals=2)
            self.ui.label_speed.setText(f'{speed}/s')
            self.ui.label_speed.setVisible(True)
        if time:
            time = round(time, 2)
            self.ui.label_time.setText(f"{time} s")
            self.ui.label_time.setVisible(True)
        if time_cpu:
            time_cpu = round(time_cpu, 2)
            self.ui.label_time_cpu.setText(f"{time_cpu} s")
            self.ui.label_time_cpu.setVisible(True)


def get_app_icon() -> QIcon:
    path = importlib.resources.files('usb_imager.resources')
    icon = QIcon(str(path) + '/' + 'usb-imager.ico')
    return icon


def get_qss_text(qss_filename: str) -> str:
    path = importlib.resources.files('usb_imager.resources')
    qss_text = path.joinpath(qss_filename).read_text()
    return qss_text



# -*- coding: utf-8 -*-
"""Timer class."""

import time


class Timer:
    """Timer in seconds."""

    def __init__(self):

        self.start_time = 0.0
        self.stop_time = 0.0
        self.start_time_cpu = 0.0
        self.stop_time_cpu = 0.0

    @property
    def result(self) -> str:
        return self.stop_time - self.start_time

    @property
    def result_cpu(self) -> str:
        return self.stop_time_cpu - self.start_time_cpu

    def start(self) -> None:
        self.start_time = time.time()
        self.start_time_cpu = time.process_time()

    def stop(self) -> None:
        self.stop_time = time.time()
        self.stop_time_cpu = time.process_time()

    def reset(self) -> None:
        self.start_time = 0.0
        self.stop_time = 0.0
        self.start_time_cpu = 0.0
        self.stop_time_cpu = 0.0



# -*- coding: utf-8 -*-
"""MessageBox wrapper."""

import os
import sys


DETECTED_GTK = []


# GTK
try:
    import gi
    gi.require_version('Gtk', '3.0')
    from gi.repository import Gtk
    DETECTED_GTK.append('gi')
except ImportError:
    pass

# QT -> PyQt
try:
    from PyQt6.QtWidgets import QApplication, QMessageBox
    DETECTED_GTK.append('pyqt')
except ImportError:
    try:
        from PyQt6.QtWidgets import QApplication, QMessageBox
        DETECTED_GTK.append('pyqt')
    except ImportError:
        pass

# QT -> PySide
try:
    from PySide6.QtWidgets import QApplication, QMessageBox
    DETECTED_GTK.append('pyside')
except ImportError:
    try:
        from PySide6.QtWidgets import QApplication, QMessageBox
        DETECTED_GTK.append('pyside')
    except ImportError:
        pass

# TkInter
try:
    from tkinter import Tk, messagebox
    DETECTED_GTK.append('tkinter')
except ImportError:
    pass

# wxPython
try:
    import wx
    DETECTED_GTK.append('wxpython')
except ImportError:
    pass


class MsgBox:
    """
    Print GUI-Messages before an application is started.

    MessageBox wrapper supports:

        GTK >= 3
        PyQt >=6
        PySide >= 6
        TkInter
        wxPython

    """

    Supported_gtk = ['gi', 'pyqt', 'pyside', 'tkinter', 'wxpython']

    def __init__(self, gtk_name: str = None):

        if gtk_name:
            if gtk_name in self.Supported_gtk:
                self.__gtk_type = gtk_name
            else:
                raise ValueError(
                    f"Valid values are {self.Supported_gtk}.")
        else:
            if DETECTED_GTK:
                self.__gtk_type = DETECTED_GTK[0]

    def information(self, message: str) -> None:
        """Information message."""
        self.__caller("Information", message)

    def warning(self, message: str) -> None:
        """Warning message."""
        self.__caller("Warning", message)

    def error(self, message: str) -> None:
        """Error message."""
        self.__caller("Error", message)

    def __caller(self, title: str, message: str) -> None:

        desktop = bool(os.getenv('DESKTOP_SESSION'))

        if desktop:
            if self.__gtk_type == 'gi':
                self.__gi(title, message)
            elif self.__gtk_type == 'pyqt' or self.__gtk_type == 'pyside':
                self.__qt(title, message)
            elif self.__gtk_type == 'tkinter':
                self.__tk(title, message)
            elif self.__gtk_type == 'wxpython':
                self.__wx(title, message)
            else:
                self.__cli(title, message)

    @staticmethod
    def __gi(title: str, message: str) -> None:

        # BUG -> No icons are displayed in the dialog box.

        if title == "Warning":
            msg_type = Gtk.MessageType.WARNING
        elif title == "Error":
            msg_type = Gtk.MessageType.ERROR
        else:
            msg_type = Gtk.MessageType.INFO

        dialog = Gtk.MessageDialog(
            title=title,
            message_type=msg_type,
            buttons=Gtk.ButtonsType.OK,
            text=message
            )

        dialog.run()
        dialog.destroy()

        # This is necessary to remove the dialog from the screen
        # after destroying it. (Standalone messagebox)
        while Gtk.events_pending():
            Gtk.main_iteration()

    @staticmethod
    def __qt(title: str, message: str) -> None:

        if QApplication.instance() is None:
            app = QApplication(sys.argv)

        if title == "Warning":
            QMessageBox.warning(None, title, message)
        elif title == "Error":
            QMessageBox.critical(None, title, message)
        else:
            QMessageBox.information(None, title, message)

    @staticmethod
    def __tk(title: str, message: str) -> None:

        root = Tk()
        root.withdraw()

        if title == "Warning":
            messagebox.showwarning(title, message)
        elif title == "Error":
            messagebox.showerror(title, message)
        else:
            messagebox.showinfo(title, message)

        root.destroy()

    @staticmethod
    def __wx(title: str, message: str) -> None:

        # BUG -> No icons are displayed in the dialog box.

        if title == "Warning":
            icon = wx.ICON_WARNING
        elif title == "Error":
            icon = wx.ICON_ERROR
        else:
            icon = wx.ICON_INFORMATION

        if wx.App.Get() is None:
            app = wx.App()

        wx.MessageBox(message, caption=title, style=icon | wx.CENTRE | wx.OK)

        # This is necessary to remove the dialog from the screen
        # after destroying it. (Standalone messagebox)
        frame = wx.Frame(None)
        frame.Destroy()

    @staticmethod
    def __cli(title: str, message: str) -> None:

        if title == "Error":
            print(f"{title}: {message}", file=sys.stderr)
        else:
            print(f"{title}: {message}")


# Testing
if __name__ == "__main__":
    msgbox = MsgBox('tkinter')
    msgbox.error(message="Programm only supports Unix!")



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'mainwindowAjfaLn.ui'
##
## Created by: Qt User Interface Compiler version 6.4.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QHBoxLayout, QLabel,
    QMainWindow, QMenu, QMenuBar, QProgressBar,
    QPushButton, QScrollArea, QSizePolicy, QSpacerItem,
    QStatusBar, QToolButton, QVBoxLayout, QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(500, 500)
        MainWindow.setMinimumSize(QSize(450, 0))
        self.actionAbout = QAction(MainWindow)
        self.actionAbout.setObjectName(u"actionAbout")
        self.actionAboutQt = QAction(MainWindow)
        self.actionAboutQt.setObjectName(u"actionAboutQt")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout = QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.checkBox_select_all = QCheckBox(self.centralwidget)
        self.checkBox_select_all.setObjectName(u"checkBox_select_all")
        self.checkBox_select_all.setChecked(True)

        self.horizontalLayout_2.addWidget(self.checkBox_select_all)

        self.checkBox_verify_all = QCheckBox(self.centralwidget)
        self.checkBox_verify_all.setObjectName(u"checkBox_verify_all")

        self.horizontalLayout_2.addWidget(self.checkBox_verify_all)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.toolButton_menu = QToolButton(self.centralwidget)
        self.toolButton_menu.setObjectName(u"toolButton_menu")
        icon = QIcon()
        iconThemeName = u"help-about-symbolic"
        if QIcon.hasThemeIcon(iconThemeName):
            icon = QIcon.fromTheme(iconThemeName)
        else:
            icon.addFile(u"../../../../../../.designer/backup", QSize(), QIcon.Normal, QIcon.Off)

        self.toolButton_menu.setIcon(icon)
        self.toolButton_menu.setPopupMode(QToolButton.InstantPopup)
        self.toolButton_menu.setToolButtonStyle(Qt.ToolButtonFollowStyle)

        self.horizontalLayout_2.addWidget(self.toolButton_menu)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.scrollArea = QScrollArea(self.centralwidget)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 480, 18))
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollAreaWidgetContents.sizePolicy().hasHeightForWidth())
        self.scrollAreaWidgetContents.setSizePolicy(sizePolicy)
        self.verticalLayout_targets = QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_targets.setSpacing(12)
        self.verticalLayout_targets.setObjectName(u"verticalLayout_targets")
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.verticalLayout.addWidget(self.scrollArea)

        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.pushButton_open = QPushButton(self.centralwidget)
        self.pushButton_open.setObjectName(u"pushButton_open")
        icon1 = QIcon()
        iconThemeName = u"document-open"
        if QIcon.hasThemeIcon(iconThemeName):
            icon1 = QIcon.fromTheme(iconThemeName)
        else:
            icon1.addFile(u"../../../../../../.designer/backup", QSize(), QIcon.Normal, QIcon.Off)

        self.pushButton_open.setIcon(icon1)

        self.horizontalLayout_3.addWidget(self.pushButton_open)

        self.label_imagename = QLabel(self.centralwidget)
        self.label_imagename.setObjectName(u"label_imagename")
        sizePolicy1 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.label_imagename.sizePolicy().hasHeightForWidth())
        self.label_imagename.setSizePolicy(sizePolicy1)

        self.horizontalLayout_3.addWidget(self.label_imagename)

        self.label_imagesize = QLabel(self.centralwidget)
        self.label_imagesize.setObjectName(u"label_imagesize")
        self.label_imagesize.setAlignment(Qt.AlignCenter)

        self.horizontalLayout_3.addWidget(self.label_imagesize)


        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.label = QLabel(self.centralwidget)
        self.label.setObjectName(u"label")

        self.horizontalLayout_5.addWidget(self.label)

        self.progressbar_all = QProgressBar(self.centralwidget)
        self.progressbar_all.setObjectName(u"progressbar_all")
        self.progressbar_all.setValue(0)

        self.horizontalLayout_5.addWidget(self.progressbar_all)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.pushButton_stop_all = QPushButton(self.centralwidget)
        self.pushButton_stop_all.setObjectName(u"pushButton_stop_all")
        icon2 = QIcon()
        iconThemeName = u"media-playback-stop"
        if QIcon.hasThemeIcon(iconThemeName):
            icon2 = QIcon.fromTheme(iconThemeName)
        else:
            icon2.addFile(u"../../../../../../.designer/backup", QSize(), QIcon.Normal, QIcon.Off)

        self.pushButton_stop_all.setIcon(icon2)

        self.horizontalLayout.addWidget(self.pushButton_stop_all)

        self.pushButton_start_all = QPushButton(self.centralwidget)
        self.pushButton_start_all.setObjectName(u"pushButton_start_all")
        icon3 = QIcon()
        iconThemeName = u"media-playback-start"
        if QIcon.hasThemeIcon(iconThemeName):
            icon3 = QIcon.fromTheme(iconThemeName)
        else:
            icon3.addFile(u"../../../../../../.designer/backup", QSize(), QIcon.Normal, QIcon.Off)

        self.pushButton_start_all.setIcon(icon3)

        self.horizontalLayout.addWidget(self.pushButton_start_all)


        self.horizontalLayout_5.addLayout(self.horizontalLayout)


        self.verticalLayout_2.addLayout(self.horizontalLayout_5)


        self.verticalLayout.addLayout(self.verticalLayout_2)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 500, 22))
        self.menubar.setLayoutDirection(Qt.RightToLeft)
        self.menuInfo = QMenu(self.menubar)
        self.menuInfo.setObjectName(u"menuInfo")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuInfo.menuAction())
        self.menuInfo.addAction(self.actionAbout)
        self.menuInfo.addAction(self.actionAboutQt)

        self.retranslateUi(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.actionAbout.setText(QCoreApplication.translate("MainWindow", u"About", None))
        self.actionAboutQt.setText(QCoreApplication.translate("MainWindow", u"About Qt", None))
#if QT_CONFIG(tooltip)
        self.checkBox_select_all.setToolTip(QCoreApplication.translate("MainWindow", u"Select all USB sticks for writing", None))
#endif // QT_CONFIG(tooltip)
        self.checkBox_select_all.setText(QCoreApplication.translate("MainWindow", u"Select all", None))
#if QT_CONFIG(tooltip)
        self.checkBox_verify_all.setToolTip(QCoreApplication.translate("MainWindow", u"Select all USB sticks for validation", None))
#endif // QT_CONFIG(tooltip)
        self.checkBox_verify_all.setText(QCoreApplication.translate("MainWindow", u"Verify all", None))
#if QT_CONFIG(tooltip)
        self.toolButton_menu.setToolTip(QCoreApplication.translate("MainWindow", u"About", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.pushButton_open.setToolTip(QCoreApplication.translate("MainWindow", u"Open image", None))
#endif // QT_CONFIG(tooltip)
        self.pushButton_open.setText(QCoreApplication.translate("MainWindow", u"Open", None))
#if QT_CONFIG(tooltip)
        self.label_imagename.setToolTip(QCoreApplication.translate("MainWindow", u"Image name", None))
#endif // QT_CONFIG(tooltip)
        self.label_imagename.setText(QCoreApplication.translate("MainWindow", u"No image", None))
#if QT_CONFIG(tooltip)
        self.label_imagesize.setToolTip(QCoreApplication.translate("MainWindow", u"Image size", None))
#endif // QT_CONFIG(tooltip)
        self.label_imagesize.setText(QCoreApplication.translate("MainWindow", u"0 GB", None))
        self.label.setText(QCoreApplication.translate("MainWindow", u"Total:", None))
#if QT_CONFIG(tooltip)
        self.progressbar_all.setToolTip(QCoreApplication.translate("MainWindow", u"Total progress", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.pushButton_stop_all.setToolTip(QCoreApplication.translate("MainWindow", u"Stop all", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.pushButton_start_all.setToolTip(QCoreApplication.translate("MainWindow", u"Start all", None))
#endif // QT_CONFIG(tooltip)
        self.menuInfo.setTitle(QCoreApplication.translate("MainWindow", u"Info", None))
    # retranslateUi




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Application informations."""

APP_NAME = "usb-imager"
APP_TITLE = "USB-Imager"
APP_DESCRIPTION = \
    "GUI-Application to write bootable disk images to USB key."

APP_LICENSE = "GPLv3"
APP_COPYRIGHT = "© 2021-2022"

APP_VERSION = "2.1.0"

APP_ORGANISATION = "Secu-Design®"
APP_ID = "com.secu-design.app.usb-imager"
APP_AUTHOR = "Skynet-Devel"
APP_AUTHOR_EMAIL = "skynet-devel.tech@secu-design.com"

APP_URL_GITLAB = "https://gitlab.com/secu-design/usb-imager"
APP_URL_PYPI = "https://pypi.org/project/usb-imager"

APP_ABOUT = f"""\
Copyright {APP_COPYRIGHT} by {APP_ORGANISATION}.
Released under the {APP_LICENSE} license.

Author:  {APP_AUTHOR}
E-mail:  {APP_AUTHOR_EMAIL}

Websites:
{APP_URL_GITLAB}
{APP_URL_PYPI}

Please send me feedback if you like
USB-Imager or if you find bugs.
"""
